package com.ruoyi.demo.service.impl;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.lang.Dict;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ruoyi.demo.entity.*;
import com.ruoyi.demo.entity.bo.*;
import com.ruoyi.demo.entity.vo.*;
import com.ruoyi.demo.mapper.*;
import com.ruoyi.demo.service.PlanService;
import com.ruoyi.demo.utils.OnStateUtil;
import com.ruoyi.demo.utils.SendCallBackUtil;
import com.ruoyi.demo.utils.SignUtil;
import com.ruoyi.demo.utils.StateUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.*;

@Service
@Transactional
public class PlanServiceImpl implements PlanService {
    @Autowired
    PlanMapper planMapper;
    @Autowired
    PlanItemMapper planItemMapper;
    @Autowired
    PlanAuditMapper planAuditMapper;
    @Autowired
    AppMapper appMapper;
    @Autowired
    CommercialMapper commercialMapper;
    @Autowired
    SystemSetterMapper systemSetterMapper;
    @Autowired
    StoreMapper storeMapper;
    @Autowired
    PlanPushStatisticsMapper planPushStatisticsMapper;
    @Autowired
    AddrCategoryMapper addrCategoryMapper;
    @Autowired
    EquipmentMapper equipmentMapper;
    @Autowired
    GoodsMapper goodsMapper;
    @Autowired
    StatisticsMapper statisticsMapper;
    @Autowired
    ExposureMapper exposureMapper;

    @Override
    public String getSign(PlanAddBo bo) {
        App app = appMapper.selectOne(new QueryWrapper<App>()
            .eq("appkey", bo.getAppkey())
            .eq("state", 1)
            .eq("deleted", "0"));
        if (app == null)
            return "error";
        Map<String, Object> params = Dict.create()
            .set("planOtherId", bo.getPlanOtherId())
            .set("appkey", bo.getAppkey())
            .set("goodsId", bo.getGoodsId())
            .set("goodsImg", bo.getGoodsImg())
            .set("goodsIntroduce", bo.getGoodsIntroduce())
            .set("goodsLogo", bo.getGoodsLogo())
            .set("goodsName", bo.getGoodsName())
            .set("goodsClassify", bo.getGoodsClassify())
            .set("goodsPrice", bo.getGoodsPrice())
            .set("items", bo.getItems())
            .set("planEndTime", bo.getPlanEndTime())
            .set("planStartTime", bo.getPlanStartTime())
            .set("sign", null)
            .set("vendorId", bo.getVendorId())
            .set("vendorName", bo.getVendorName());
        return SignUtil.sign(params, app.getAppSecret());
    }

    @Override
    public String addPlan(PlanAddBo bo) {
        // app是否可用
        App app = appMapper.selectOne(new QueryWrapper<App>()
            .eq("appkey", bo.getAppkey())
            .eq("state", 1)
            .eq("deleted", "0"));
        if (app == null)
            return "-1";
        // 校验
        Map<String, Object> params = Dict.create()
            .set("planOtherId", bo.getPlanOtherId())
            .set("appkey", bo.getAppkey())
            .set("goodsId", bo.getGoodsId())
            .set("goodsImg", bo.getGoodsImg())
            .set("goodsIntroduce", bo.getGoodsIntroduce())
            .set("goodsLogo", bo.getGoodsLogo())
            .set("goodsName", bo.getGoodsName())
            .set("goodsClassify", bo.getGoodsClassify())
            .set("goodsPrice", bo.getGoodsPrice())
            .set("items", bo.getItems())
            .set("planEndTime", bo.getPlanEndTime())
            .set("planStartTime", bo.getPlanStartTime())
            .set("sign", bo.getSign())
            .set("vendorId", bo.getVendorId())
            .set("vendorName", bo.getVendorName());
        if (!SignUtil.verify(params, app.getAppSecret()))
            return "-3";
        // 插入广告计划
        Plan plan = new Plan(app, bo);
        int i = planMapper.insert(plan);
        if (i == 0)
            return "0";
        // 插入广告计划明细
        for (PlanItemAddBo itemAddBo : bo.getItems()) {
            PlanItem item = new PlanItem(plan.getPlanId(), itemAddBo);
            planItemMapper.insert(item);
        }
        // 插入商品信息
        Goods query = goodsMapper.selectOne(new QueryWrapper<Goods>()
            .eq("appkey", bo.getAppkey())
            .eq("goods_id", bo.getGoodsId())
            .eq("vendor_id", bo.getVendorId()));
        if (query == null)
            goodsMapper.insert(new Goods(plan));
        return plan.getPlanId();
    }

    @Override
    public int deletePlan(String id) {
        planItemMapper.delete(new QueryWrapper<PlanItem>()
            .eq("plan_id", id));
        return planMapper.delete(new QueryWrapper<Plan>()
            .eq("plan_id", id));
    }

    @Override
    public int deletePlans(List<String> ids) {
        int i = 0;
        for (String id : ids) {
            i += planMapper.delete(new QueryWrapper<Plan>()
                .eq("plan_id", id));
        }
        return i;
    }

    @Override
    public int checkPlan(PlanBo bo, Long userId, String userName) {
        // 是否有该计划
        Plan plan = planMapper.selectOne(new QueryWrapper<Plan>()
            .eq("plan_id", bo.getPlanId())
            .eq("deleted", "0"));
        if (plan == null)
            return -1;
        // 没有该app或者该app已经被禁用，则不允许审核
        App app = appMapper.selectOne(new QueryWrapper<App>()
            .eq("appkey", plan.getAppkey())
            .eq("state", 1)
            .eq("deleted", "0"));
        if (app == null)
            return -1;
        // 确定审核状态并更新广告计划
        plan.setState(bo.getState());
        Date now = DateUtil.date(); // 启动回调地址时也用该时间
        plan.setAuditTime(now);
        int i = planMapper.updateById(plan);
        if (i == 0) return 0;
        // 插入审核记录
        PlanAudit audit = new PlanAudit(bo, userId.toString(), userName);
        int insert = planAuditMapper.insert(audit);
        if (insert == 1) { // 审核计划插入成功
            // 启动回调地址
            CallBackBean callBackBean = new CallBackBean();
            callBackBean.setAuditPerson(userName);
            if (bo.getState() == 1)
                callBackBean.setAuditComment("审核通过");
            else callBackBean.setAuditComment("审核不通过");
            callBackBean.setAuditTime(now);
            try {
                SendCallBackUtil.sendCallBack(app.getCallBack(), callBackBean);
            } catch (Exception e) {
                return -2;
            }
        }
        return insert;
    }

    @Override
    public MyPage<PlanListVo> getPlanList(PlanBo bo) {
        QueryWrapper<Plan> planQueryWrapper = new QueryWrapper<>();
        if (bo.getText() != null)
            planQueryWrapper.and(planQueryWrapper1 -> {
                planQueryWrapper1/*.like("plan_id",bo.getText())
                    .or()*/
                    .like("plan_name", bo.getText())
                    .or()
                    .like("principal_name", bo.getText())
                    .or()
                    .like("principal_telephone", bo.getText());
            });
        if (bo.getStartTime() != null) // 搜索开始时间小于等于广告计划创建时间
            planQueryWrapper.apply("plan_create_time >= STR_TO_DATE({0},'%Y-%m-%d %H:%i:%s')", bo.getStartTime());
        if (bo.getEndTime() != null) // 搜索结束时间大于等于广告计划创建时间
            planQueryWrapper.apply("plan_create_time <= STR_TO_DATE({0},'%Y-%m-%d %H:%i:%s')", bo.getEndTime());
        planQueryWrapper.eq("state", StateUtil.APPROVED)
            .eq("deleted", "0")
            .orderByDesc("plan_create_time");
        Page<Plan> page = new Page<>(bo.getPageNum(), bo.getPageSize());
        Page<Plan> planPage = planMapper.selectPage(page, planQueryWrapper);
        List<Plan> plans = planPage.getRecords();
        List<PlanListVo> vos = new ArrayList<>();
        for (Plan plan : plans) {
            vos.add(new PlanListVo(plan));
        }
        return new MyPage<>(page, vos);
    }

    @Override
    public PlanDetailVo getPlanDetail(String planId) {
        Plan plan = planMapper.selectOne(new QueryWrapper<Plan>()
            .eq("plan_id", planId)
            .eq("deleted", "0"));
        if (plan == null)
            return null;
        List<PlanItem> items = planItemMapper.selectList(new QueryWrapper<PlanItem>()
            .eq("plan_id", planId));
        List<PlanItemVo> itemVos = new ArrayList<>();
        if (items != null)
            for (PlanItem item : items) {
                itemVos.add(new PlanItemVo(item));
            }
        PlanDetailVo vo = new PlanDetailVo(plan, itemVos);
        return vo;
    }

    @Override
    public List<PlanPushVo> getPlanPush(String equipmentId) {
//        根据设备id查询可投放的广告计划
//        再根据每个广告计划id，查询其商品信息
        List<PlanPushVo> pushVos = new ArrayList<>();
        List<PlanItem> items = planItemMapper.selectList(new QueryWrapper<PlanItem>()
            .eq("equipment_id", equipmentId)
            .eq("deleted", "0"));
        for (PlanItem item : items) {
            Plan plan = planMapper.selectOne(new QueryWrapper<Plan>()
                // 广告计划开始时间 <= 当前时间 < 广告计划结束时间
                .apply("plan_start_time <= STR_TO_DATE({0},'%Y-%m-%d %H:%i:%s')", DateUtil.date())
                .apply("plan_end_time > STR_TO_DATE({0},'%Y-%m-%d %H:%i:%s')", DateUtil.date())
                .eq("plan_id", item.getPlanId())
                .eq("state", StateUtil.APPROVED)
                .eq("deleted", "0"));
            if (plan != null)
                pushVos.add(new PlanPushVo(item, plan));
        }
        return pushVos;
    }

    @Override
    public List<PlanPushVo> getPlanPushByCode(String equipmentCode) {
        List<PlanPushVo> pushVos = new ArrayList<>();
        // 根据设备编号查询设备下的广告计划子项
        List<PlanItem> items = planItemMapper.selectList(new QueryWrapper<PlanItem>()
            .eq("equipment_code", equipmentCode)
            .eq("deleted", "0"));
        // 排序
        items.sort(Comparator.comparing(PlanItem::getSort).reversed());
        for (PlanItem item : items) { // 根据计划子项查询计划，并补齐返回信息
            Plan plan = planMapper.selectOne(new QueryWrapper<Plan>()
                .eq("plan_id", item.getPlanId())
                // 广告计划开始时间 <= 当前时间 < 广告计划结束时间
                .apply("plan_start_time <= STR_TO_DATE({0},'%Y-%m-%d %H:%i:%s')", DateUtil.date())
                .apply("plan_end_time > STR_TO_DATE({0},'%Y-%m-%d %H:%i:%s')", DateUtil.date())
                .eq("state", StateUtil.APPROVED)
                .eq("on_state", OnStateUtil.UP)
                .eq("deleted", "0"));
            if (plan != null)
                pushVos.add(new PlanPushVo(item, plan));
        }
        return pushVos;
    }

    @Override
    public int planUpDown(String planId) {
        // 是否存在该广告计划
        Plan plan = planMapper.selectOne(new QueryWrapper<Plan>()
            .eq("plan_id", planId)
            .eq("deleted", "0"));
        if (plan == null)
            return -1;
        // 改变状态，1：上架；0：下架
        plan.setOnState(plan.getOnState() + 1);
        if (plan.getOnState() >= 2) {
            plan.setOnState(0);
            // 发送下架信息
            try {
                App app = appMapper.selectOne(new QueryWrapper<App>()
                    .eq("appkey", plan.getAppkey())
                    .eq("state", 1)
                    .eq("deleted", "0"));
                Map<String, Object> map = Dict.create()
                    .set("msg", "您的广告\"" + plan.getPlanName() + "\"已下架")
                    .set("time", DateUtil.date());
                SendCallBackUtil.sendCallBack(app.getCallBack(), map);
            } catch (Exception e) {
            }
        }
        return planMapper.updateById(plan);
    }

    @Override
    public int planPushUpDown(UpDownBo bo) {
        // app是否可用
        App app = appMapper.selectOne(new QueryWrapper<App>()
            .eq("appkey", bo.getAppkey())
            .eq("state", 1)
            .eq("deleted", "0"));
        if (app == null)
            return -1;
        // 校验
        Map<String, Object> params = Dict.create()
            .set("appkey", bo.getAppkey())
            .set("planIds", bo.getPlanIds())
            .set("sign", bo.getSign());
        if (!SignUtil.verify(params, app.getAppSecret()))
            return -3;
        for (String planId : bo.getPlanIds()){
            Plan plan = planMapper.selectOne(new QueryWrapper<Plan>()
                .eq("plan_id", planId)
                .eq("deleted", "0"));
            if (plan == null)
                continue;
            // 改变状态，1：上架；0：下架
            plan.setOnState(plan.getOnState() + 1);
            if (plan.getOnState() >= 2) {
                plan.setOnState(0);
                // 发送下架信息
                try {
                    Map<String, Object> map = Dict.create()
                        .set("msg", "您的广告\"" + plan.getPlanName() + "\"已下架")
                        .set("time", DateUtil.date());
                    SendCallBackUtil.sendCallBack(app.getCallBack(), map);
                } catch (Exception e) {
                }
            }
            planMapper.updateById(plan);
        }
        return 1;
    }

    @Override
    public String pushGetSign(UpDownBo bo) {
        // app是否可用
        App app = appMapper.selectOne(new QueryWrapper<App>()
            .eq("appkey", bo.getAppkey())
            .eq("state", 1)
            .eq("deleted", "0"));
        if (app == null)
            return "error";
        // 校验
        Map<String, Object> params = Dict.create()
            .set("appkey", bo.getAppkey())
            .set("planIds", bo.getPlanIds())
            .set("sign", bo.getSign());
        return SignUtil.sign(params, app.getAppSecret());
    }

    @Override
    public MyPage<PlanManageListVo> getPlanManageList(PlanBo bo) {
        QueryWrapper<Plan> planQueryWrapper = new QueryWrapper<>();
        planQueryWrapper.eq("state", bo.getState());
        if (bo.getText() != null)
            planQueryWrapper.and(planQueryWrapper1 -> {
                planQueryWrapper1/*.like("plan_id",bo.getText())
                    .or()*/
                    .like("plan_name", bo.getText())
                    .or()
                    .like("app_name", bo.getText())
                    .or()
                    .like("commercial_name", bo.getText());
            });
        if (bo.getStartTime() != null)
            planQueryWrapper.apply("plan_start_time >= STR_TO_DATE({0},'%Y-%m-%d %H:%i:%s')", bo.getStartTime());
        if (bo.getEndTime() != null)
            planQueryWrapper.apply("plan_end_time <= STR_TO_DATE({0},'%Y-%m-%d %H:%i:%s')", bo.getEndTime());
        planQueryWrapper.eq("deleted", "0")
            .orderByDesc("plan_create_time");
        Page<Plan> page = new Page<>(bo.getPageNum(), bo.getPageSize());
        List<Plan> plans = planMapper.selectPage(page, planQueryWrapper).getRecords();
        List<PlanManageListVo> vos = new ArrayList<>();
        for (Plan plan : plans) {
            PlanManageListVo vo = new PlanManageListVo(plan);
            // 查询计划的设备数量
            Integer count = planItemMapper.selectEquipmentCountByPlanId(plan.getPlanId());
            vo.setEquipmentCount(count);
            vos.add(vo);
        }
        return new MyPage<>(page, vos);
    }

    @Override
    public PlanManageDetailVo getPlanManageDetail(String planId) {
        Plan plan = planMapper.selectOne(new QueryWrapper<Plan>()
            .eq("plan_id", planId)
            .eq("deleted", "0"));
        if (plan == null)
            return null;
        List<PlanItem> items = planItemMapper.selectList(new QueryWrapper<PlanItem>()
            .eq("plan_id", planId)
            .eq("deleted", "0"));
        List<PlanManageItemVo> itemVos = new ArrayList<>();
        for (PlanItem item : items) { // 补齐店铺信息
            PlanManageItemVo itemVo = new PlanManageItemVo(item);
            if (item.getStoreId() != null) {
                Store store = storeMapper.selectById(item.getStoreId());
                if (store != null)
                    itemVo.setStoreName(store.getStoreName());
            }
            itemVos.add(itemVo);
        }
        PlanManageDetailVo vo = new PlanManageDetailVo(plan, itemVos);
        return vo;
    }

    @Override
    public MyPage<PlanPushListVo> getPlanPushList(PlanBo bo) {
        QueryWrapper<Plan> planQueryWrapper = new QueryWrapper<>();
        planQueryWrapper.eq("appkey", bo.getAppkey());
//        planQueryWrapper.eq("commercial_id",bo.getCommercialId());
        if (bo.getText() != null)
            planQueryWrapper.and(planQueryWrapper1 -> {
                planQueryWrapper1.like("plan_id", bo.getText())
                    .or()
                    .like("plan_name", bo.getText())
                    .or()
                    .like("principal_name", bo.getText())
                    .or()
                    .like("principal_telephone", bo.getText());
            });
        if (bo.getStartTime() != null)
            planQueryWrapper.apply("plan_start_time >= STR_TO_DATE({0},'%Y-%m-%d %H:%i:%s')", bo.getStartTime());
        if (bo.getEndTime() != null)
            planQueryWrapper.apply("plan_end_time <= STR_TO_DATE({0},'%Y-%m-%d %H:%i:%s')", bo.getEndTime());
        planQueryWrapper.eq("deleted", "0")
            .orderByDesc("plan_create_time");
        Page<Plan> page = new Page<>(bo.getPageNum(), bo.getPageSize());
        List<Plan> plans = planMapper.selectPage(page, planQueryWrapper).getRecords();
        List<PlanPushListVo> vos = new ArrayList<>();
        for (Plan plan : plans) {
            PlanPushListVo vo = new PlanPushListVo(plan);
            // 根据计划id获取设备数量
            Integer count = planItemMapper.selectEquipmentCountByPlanId(plan.getPlanId());
            vo.setEquipmentCount(count);
            vos.add(vo);
        }
        return new MyPage<>(page, vos);
    }

    @Override
    public MyPage<PlanPushStatisticsListVo> getPlanPushStatisticsList(PlanBo bo) {
        QueryWrapper<PlanPushStatistics> statisticsQueryWrapper = new QueryWrapper<>();
        if (bo.getText() != null)
            statisticsQueryWrapper.and(planPushStatisticsQueryWrapper -> {
                planPushStatisticsQueryWrapper/*.like("equipment_id",bo.getText())
                    .or()
                    .like("equipment_code",bo.getText())
                    .or()*/
                    .like("equipment_name", bo.getText());
            });
        if (bo.getStartTime() != null)
            statisticsQueryWrapper.apply("laster_time >= STR_TO_DATE({0},'%Y-%m-%d %H:%i:%s')", bo.getStartTime());
        if (bo.getEndTime() != null)
            statisticsQueryWrapper.apply("laster_time <= STR_TO_DATE({0},'%Y-%m-%d %H:%i:%s')", bo.getEndTime());
        Page<PlanPushStatistics> page = new Page<>(bo.getPageNum(), bo.getPageSize());
        List<PlanPushStatistics> list = planPushStatisticsMapper.selectPage(page, statisticsQueryWrapper).getRecords();
        List<PlanPushStatisticsListVo> vos = new ArrayList<>();
        for (PlanPushStatistics statistics : list) {
            vos.add(new PlanPushStatisticsListVo(statistics));
        }
        return new MyPage<>(page, vos);
    }

    @Override
    public void planPushStatisticsStart() {
        // 清除当前月的统计记录
        QueryWrapper<PlanPushStatistics> planPushStatisticsQueryWrapper = new QueryWrapper<>();
        planPushStatisticsQueryWrapper.apply("DATE_FORMAT(laster_time,'%Y-%m') = DATE_FORMAT({0},'%Y-%m')", DateUtil.date());
        planPushStatisticsMapper.delete(planPushStatisticsQueryWrapper);
        // 获取所有设备，遍历
        for (Equipment equipment : equipmentMapper.selectList(null)) {
            // 先查询设备下所有plan_item，再根据item_id统计查询曝光记录表获取曝光次数
            List<String> ids = planItemMapper.getItemIdsByCode(equipment.getEquipmentCode());
            Integer carousalCount = exposureMapper.selectTotalCountByItemIds(ids);
            // 统计查询统计信息表获取累计扫码量、累计交易数、累计用户数、累计收入
            Integer scanCount = statisticsMapper.selectScanByEquipmentCode(equipment.getEquipmentCode());
            Integer tradeCount = statisticsMapper.selectDealCountByEquipmentCode(equipment.getEquipmentCode());
            List<String> users = statisticsMapper.selectUserIdsByEquipmentCode(equipment.getEquipmentCode());
            BigDecimal totalPrice = statisticsMapper.selectTotalPriceByEquipmentCode(equipment.getEquipmentCode());
            // 插入记录
            PlanPushStatistics planPushStatistics = new PlanPushStatistics();
            planPushStatistics.setEquipmentId(equipment.getEquipmentId());
            planPushStatistics.setEquipmentName(equipment.getEquipmentName());
            planPushStatistics.setEquipmentCode(equipment.getEquipmentCode());
            planPushStatistics.setCarouselCount(carousalCount);
            planPushStatistics.setScanCodeCount(scanCount);
            planPushStatistics.setTradeCount(tradeCount);
            planPushStatistics.setTotalUserCount(users == null ? 0 : users.size());
            planPushStatistics.setTotalPrice(totalPrice == null ? BigDecimal.ZERO : totalPrice);
            planPushStatistics.setLasterTime(DateUtil.date());
            planPushStatisticsMapper.insert(planPushStatistics);
        }
    }

    @Override
    public PlanHomeVo getPlanTotal() {
        PlanHomeVo vo = new PlanHomeVo();
        vo.setTotal(planMapper.selectPlanTotal());
        return vo;
    }

    @Override
    public List<AddrCategory> getAddrCategory() {
        return addrCategoryMapper.selectList(null);
    }

    @Override
    public List<AddrCategory> getAddrCategoryLevel(String code, String level) {
        if ("province".equals(level)) {
            return addrCategoryMapper.selectList(new QueryWrapper<AddrCategory>()
                .eq("city", ""));
        } else if ("city".equals(level)) {
            QueryWrapper<AddrCategory> queryWrapper = new QueryWrapper<>();
            queryWrapper.likeRight("addr_code", code.substring(0, 2));
            queryWrapper.and(categoryQueryWrapper -> {
                categoryQueryWrapper.eq("district", "").and(addrCategoryQueryWrapper -> {
                    addrCategoryQueryWrapper.ne("city", "");
                });
            });
            return addrCategoryMapper.selectList(queryWrapper);
        } else if ("zone".equals(level)) {
            QueryWrapper<AddrCategory> queryWrapper = new QueryWrapper<>();
            queryWrapper.likeRight("addr_code", code.substring(0, 4));
            queryWrapper.and(categoryQueryWrapper -> {
                categoryQueryWrapper.ne("district", "");
            });
            return addrCategoryMapper.selectList(queryWrapper);
        }
        return null;
    }

    @Override
    public Page<Equipment> getEquipments(PlanBo bo) {
        QueryWrapper<Equipment> equipmentQueryWrapper = new QueryWrapper<>();
        equipmentQueryWrapper.eq("state", 1);
        if (bo.getText() != null)
            equipmentQueryWrapper.and(equipmentQueryWrapper1 -> {
                equipmentQueryWrapper1.like("equipment_id", bo.getText())
                    .or()
                    .like("equipment_code", bo.getText())
                    .or()
                    .like("equipment_name", bo.getText());
            });
        if (bo.getAddrCodes() != null) {
            for (String addrCode : bo.getAddrCodes()) {
                // 拆分地址码
                String province = String.valueOf(addrCode.charAt(0)) + String.valueOf(addrCode.charAt(1));
                String city = String.valueOf(addrCode.charAt(2)) + String.valueOf(addrCode.charAt(3));
                String district = String.valueOf(addrCode.charAt(4)) + String.valueOf(addrCode.charAt(5));

                if (district.equals("00")) {
                    if (city.equals("00"))
                        equipmentQueryWrapper.or().likeRight("addr_code", province);
                    else
                        equipmentQueryWrapper.or().likeRight("addr_code", province + city);
                } else
                    equipmentQueryWrapper.or().likeRight("addr_code", province + city + district);
            }
        }
        Page<Equipment> page = new Page<>(bo.getPageNum(), bo.getPageSize());
        return equipmentMapper.selectPage(page, equipmentQueryWrapper);
    }

    @Override
    public void checkPlanTime() {
        /* 逻辑：
         每天8点检查一次，假设当前时间为[2023-1-3 08:00:00]，
         查询[2023-1-3 08:00:00 >= 计划结束时间 >= 2023-1-2 08:00:00]的计划，
         获得对应的appkey，在查询这些appkey对应的小程序，
         获得对应的callBack，对这些callBack返回信息。
        */
        List<Plan> plans = planMapper.selectList(new QueryWrapper<Plan>()
            .eq("state", StateUtil.APPROVED) // 通过审核
            .eq("on_state", OnStateUtil.UP) // 上架
            .apply("plan_end_time <= STR_TO_DATE({0},'%Y-%m-%d %H:%i:%s')", DateUtil.date().toString())
            .apply("plan_end_time >= STR_TO_DATE({0},'%Y-%m-%d %H:%i:%s')", DateUtil.offsetDay(new Date(), -1).toString()));
        for (Plan plan : plans) {
            App app = appMapper.selectOne(new QueryWrapper<App>()
                .eq("appkey", plan.getAppkey())
                .eq("state", 1) // 启用
                .eq("deleted", "0"));
            Map<String, Object> map = Dict.create()
                .set("msg", "您的广告\"" + plan.getPlanName() + "\"已下架")
                .set("time", DateUtil.date());
            try {
                SendCallBackUtil.sendCallBack(app.getCallBack(), map);
            } catch (Exception e) {
                System.out.println("[appkey:" + app.getAppkey() + ",appName:" + app.getAppName() + "]的回调地址[" + app.getCallBack() + "]失效");
            }
        }
    }
}
